#region Using declarations
using System;
using System.ComponentModel;
using System.Diagnostics;
using System.Drawing;
using System.Drawing.Drawing2D;
using System.Xml.Serialization;
using NinjaTrader.Cbi;
using NinjaTrader.Data;
using NinjaTrader.Gui.Chart;
#endregion

// This namespace holds all indicators and is required. Do not change it.
namespace NinjaTrader.Indicator
{
    /// <summary>
    /// Color each Candle
    /// </summary>
    [Description("Overrides the default Ninja Trader Candlestick Colors")]
    public class CandleColor : Indicator
    {
        #region Variables
		private Color	barColorUp 		= Color.White;
		private Color	barColorDown 	= Color.Red;
		private Color	lastBarColor	= Color.Empty;
		#endregion

        /// <summary>
        /// This method is used to configure the indicator and is called once before any bar data is loaded.
        /// </summary>
        protected override void Initialize()
        {
            CalculateOnBarClose	= false;
            Overlay				= true;
            PriceTypeSupported	= false;
        }

        /// <summary>
        /// Called on each bar update event (incoming tick)
        /// </summary>
        protected override void OnBarUpdate()
        {
			if (Open[0] == Close[0])
				BarColor = lastBarColor;
			else
				BarColor = (Close[0] > Open[0] ? BarColorUp : BarColorDown);
			
			lastBarColor = BarColor;
        }

        #region Properties
		[XmlIgnore()]
		[Description("Color of up bars.")]
		[Category("Visual")]
		[Gui.Design.DisplayNameAttribute("Up color")]
		public Color BarColorUp
		{
			get { return barColorUp; }
			set { barColorUp = value; }
		}

		[XmlIgnore()]
		[Description("Color of down bars.")]
		[Category("Visual")]
		[Gui.Design.DisplayNameAttribute("Down color")]
		public Color BarColorDown
		{
			get { return barColorDown; }
			set { barColorDown = value; }
		}

        #endregion
    }
}

#region NinjaScript generated code. Neither change nor remove.
// This namespace holds all indicators and is required. Do not change it.
namespace NinjaTrader.Indicator
{
    public partial class Indicator : IndicatorBase
    {
        private CandleColor[] cacheCandleColor = null;

        private static CandleColor checkCandleColor = new CandleColor();

        /// <summary>
        /// Overrides the default Ninja Trader Candlestick Colors
        /// </summary>
        /// <returns></returns>
        public CandleColor CandleColor()
        {
            return CandleColor(Input);
        }

        /// <summary>
        /// Overrides the default Ninja Trader Candlestick Colors
        /// </summary>
        /// <returns></returns>
        public CandleColor CandleColor(Data.IDataSeries input)
        {

            if (cacheCandleColor != null)
                for (int idx = 0; idx < cacheCandleColor.Length; idx++)
                    if (cacheCandleColor[idx].EqualsInput(input))
                        return cacheCandleColor[idx];

            CandleColor indicator = new CandleColor();
            indicator.BarsRequired = BarsRequired;
            indicator.CalculateOnBarClose = CalculateOnBarClose;
            indicator.Input = input;
            indicator.SetUp();

            CandleColor[] tmp = new CandleColor[cacheCandleColor == null ? 1 : cacheCandleColor.Length + 1];
            if (cacheCandleColor != null)
                cacheCandleColor.CopyTo(tmp, 0);
            tmp[tmp.Length - 1] = indicator;
            cacheCandleColor = tmp;
            Indicators.Add(indicator);

            return indicator;
        }

    }
}

// This namespace holds all market analyzer column definitions and is required. Do not change it.
namespace NinjaTrader.MarketAnalyzer
{
    public partial class Column : ColumnBase
    {
        /// <summary>
        /// Overrides the default Ninja Trader Candlestick Colors
        /// </summary>
        /// <returns></returns>
        [Gui.Design.WizardCondition("Indicator")]
        public Indicator.CandleColor CandleColor()
        {
            return _indicator.CandleColor(Input);
        }

        /// <summary>
        /// Overrides the default Ninja Trader Candlestick Colors
        /// </summary>
        /// <returns></returns>
        public Indicator.CandleColor CandleColor(Data.IDataSeries input)
        {
            return _indicator.CandleColor(input);
        }

    }
}

// This namespace holds all strategies and is required. Do not change it.
namespace NinjaTrader.Strategy
{
    public partial class Strategy : StrategyBase
    {
        /// <summary>
        /// Overrides the default Ninja Trader Candlestick Colors
        /// </summary>
        /// <returns></returns>
        [Gui.Design.WizardCondition("Indicator")]
        public Indicator.CandleColor CandleColor()
        {
            return _indicator.CandleColor(Input);
        }

        /// <summary>
        /// Overrides the default Ninja Trader Candlestick Colors
        /// </summary>
        /// <returns></returns>
        public Indicator.CandleColor CandleColor(Data.IDataSeries input)
        {
            if (InInitialize && input == null)
                throw new ArgumentException("You only can access an indicator with the default input/bar series from within the 'Initialize()' method");

            return _indicator.CandleColor(input);
        }

    }
}
#endregion
