#region Using declarations
using System;
using System.ComponentModel;
using System.Diagnostics;
using System.Drawing;
using System.Drawing.Drawing2D;
using System.Xml.Serialization;
using NinjaTrader.Cbi;
using NinjaTrader.Data;
using NinjaTrader.Gui.Chart;
#endregion

// This namespace holds all indicators and is required. Do not change it.
namespace NinjaTrader.Indicator
{
    /// <summary>
    /// MACD only, no Histogram
    /// </summary>
    [Description("MACD only, no Histogram")]
    public class Macd2 : Indicator
    {
        #region Variables
        // Wizard generated variables
            private int fast = 12; // Default setting for Fast
            private int slow = 26; // Default setting for Slow
            private int smooth = 9; // Default setting for Smooth
        // User defined variables (add any user defined variables below)
		private	DataSeries		fastEma;
		private	DataSeries		slowEma;
		
        #endregion

        /// <summary>
        /// This method is used to configure the indicator and is called once before any bar data is loaded.
        /// </summary>
        protected override void Initialize()
        {
            Add(new Plot(new Pen(Color.FromKnownColor(KnownColor.FloralWhite), 2), PlotStyle.Line, "MacdLine"));
            Add(new Plot(new Pen(Color.FromKnownColor(KnownColor.DodgerBlue), 2), PlotStyle.Line, "Signal"));
            Add(new Line(Color.FromKnownColor(KnownColor.Red), 2.0, "Sell"));
            Add(new Line(Color.FromKnownColor(KnownColor.Cyan), 0, "Zero"));
            Add(new Line(Color.FromKnownColor(KnownColor.LimeGreen), -2.0, "Buy"));
            CalculateOnBarClose	= false;
            Overlay				= false;
            PriceTypeSupported	= false;

			fastEma	= new DataSeries(this);
			slowEma	= new DataSeries(this);

        }

        /// <summary>
        /// Called on each bar update event (incoming tick)
        /// </summary>
        protected override void OnBarUpdate()
        {
			if (CurrentBar == 0)
			{
				fastEma.Set(Input[0]);
				slowEma.Set(Input[0]);
				MacdLine.Set(0);
				Signal.Set(0);
			}
			else
			{
				fastEma.Set((2.0 / (1 + Fast)) * Input[0] + (1 - (2.0 / (1 + Fast))) * fastEma[1]);
				slowEma.Set((2.0 / (1 + Slow)) * Input[0] + (1 - (2.0 / (1 + Slow))) * slowEma[1]);

				double macd_		= fastEma[0] - slowEma[0];
				double signal_	= (2.0 / (1 + Smooth)) * macd_ + (1 - (2.0 / (1 + Smooth))) * Signal[1];
				
				MacdLine.Set(macd_);
				Signal.Set(signal_);
			}
        }

        #region Properties
        [Browsable(false)]	// this line prevents the data series from being displayed in the indicator properties dialog, do not remove
        [XmlIgnore()]		// this line ensures that the indicator can be saved/recovered as part of a chart template, do not remove
        public DataSeries MacdLine
        {
            get { return Values[0]; }
        }

        [Browsable(false)]	// this line prevents the data series from being displayed in the indicator properties dialog, do not remove
        [XmlIgnore()]		// this line ensures that the indicator can be saved/recovered as part of a chart template, do not remove
        public DataSeries Signal
        {
            get { return Values[1]; }
        }

        [Description("fast EMA period")]
        [Category("Parameters")]
        public int Fast
        {
            get { return fast; }
            set { fast = Math.Max(1, value); }
        }

        [Description("slow EMA period")]
        [Category("Parameters")]
        public int Slow
        {
            get { return slow; }
            set { slow = Math.Max(1, value); }
        }

        [Description("smoothing")]
        [Category("Parameters")]
        public int Smooth
        {
            get { return smooth; }
            set { smooth = Math.Max(1, value); }
        }
        #endregion
    }
}

#region NinjaScript generated code. Neither change nor remove.
// This namespace holds all indicators and is required. Do not change it.
namespace NinjaTrader.Indicator
{
    public partial class Indicator : IndicatorBase
    {
        private Macd2[] cacheMacd2 = null;

        private static Macd2 checkMacd2 = new Macd2();

        /// <summary>
        /// MACD only, no Histogram
        /// </summary>
        /// <returns></returns>
        public Macd2 Macd2(int fast, int slow, int smooth)
        {
            return Macd2(Input, fast, slow, smooth);
        }

        /// <summary>
        /// MACD only, no Histogram
        /// </summary>
        /// <returns></returns>
        public Macd2 Macd2(Data.IDataSeries input, int fast, int slow, int smooth)
        {
            checkMacd2.Fast = fast;
            fast = checkMacd2.Fast;
            checkMacd2.Slow = slow;
            slow = checkMacd2.Slow;
            checkMacd2.Smooth = smooth;
            smooth = checkMacd2.Smooth;

            if (cacheMacd2 != null)
                for (int idx = 0; idx < cacheMacd2.Length; idx++)
                    if (cacheMacd2[idx].Fast == fast && cacheMacd2[idx].Slow == slow && cacheMacd2[idx].Smooth == smooth && cacheMacd2[idx].EqualsInput(input))
                        return cacheMacd2[idx];

            Macd2 indicator = new Macd2();
            indicator.BarsRequired = BarsRequired;
            indicator.CalculateOnBarClose = CalculateOnBarClose;
            indicator.Input = input;
            indicator.Fast = fast;
            indicator.Slow = slow;
            indicator.Smooth = smooth;
            indicator.SetUp();

            Macd2[] tmp = new Macd2[cacheMacd2 == null ? 1 : cacheMacd2.Length + 1];
            if (cacheMacd2 != null)
                cacheMacd2.CopyTo(tmp, 0);
            tmp[tmp.Length - 1] = indicator;
            cacheMacd2 = tmp;
            Indicators.Add(indicator);

            return indicator;
        }

    }
}

// This namespace holds all market analyzer column definitions and is required. Do not change it.
namespace NinjaTrader.MarketAnalyzer
{
    public partial class Column : ColumnBase
    {
        /// <summary>
        /// MACD only, no Histogram
        /// </summary>
        /// <returns></returns>
        [Gui.Design.WizardCondition("Indicator")]
        public Indicator.Macd2 Macd2(int fast, int slow, int smooth)
        {
            return _indicator.Macd2(Input, fast, slow, smooth);
        }

        /// <summary>
        /// MACD only, no Histogram
        /// </summary>
        /// <returns></returns>
        public Indicator.Macd2 Macd2(Data.IDataSeries input, int fast, int slow, int smooth)
        {
            return _indicator.Macd2(input, fast, slow, smooth);
        }

    }
}

// This namespace holds all strategies and is required. Do not change it.
namespace NinjaTrader.Strategy
{
    public partial class Strategy : StrategyBase
    {
        /// <summary>
        /// MACD only, no Histogram
        /// </summary>
        /// <returns></returns>
        [Gui.Design.WizardCondition("Indicator")]
        public Indicator.Macd2 Macd2(int fast, int slow, int smooth)
        {
            return _indicator.Macd2(Input, fast, slow, smooth);
        }

        /// <summary>
        /// MACD only, no Histogram
        /// </summary>
        /// <returns></returns>
        public Indicator.Macd2 Macd2(Data.IDataSeries input, int fast, int slow, int smooth)
        {
            if (InInitialize && input == null)
                throw new ArgumentException("You only can access an indicator with the default input/bar series from within the 'Initialize()' method");

            return _indicator.Macd2(input, fast, slow, smooth);
        }

    }
}
#endregion
