#region Using declarations
using System;
using System.ComponentModel;
using System.Diagnostics;
using System.Drawing;
using System.Drawing.Drawing2D;
using System.Xml.Serialization;
using NinjaTrader.Cbi;
using NinjaTrader.Data;
using NinjaTrader.Gui.Chart;
#endregion

// This namespace holds all indicators and is required. Do not change it.
namespace NinjaTrader.Indicator
{
    /// <summary>
    /// Rate of Change with Signal Line
    /// </summary>
    [Description("Rate of Change with Signal Line")]
    public class ROC1 : Indicator
    {
        #region Variables
        // Wizard generated variables
            private int rocLen = 7; // Default setting for RocLen
            private int signalLen = 5; // Default setting for SignalLen
        // User defined variables (add any user defined variables below)
        #endregion

        /// <summary>
        /// This method is used to configure the indicator and is called once before any bar data is loaded.
        /// </summary>
        protected override void Initialize()
        {
            Add(new Plot(Color.FromKnownColor(KnownColor.White), PlotStyle.Line, "RocLine"));
            Add(new Plot(Color.FromKnownColor(KnownColor.DodgerBlue), PlotStyle.Line, "SigLine"));
            Add(new Line(Color.FromKnownColor(KnownColor.DarkGray), 0, "ZeroLine"));
            CalculateOnBarClose	= true;
            Overlay				= false;
            PriceTypeSupported	= false;
			
			Plots[0].Pen.Width = 2;
			Plots[1].Pen.Width = 1;
			Lines[0].Pen.DashStyle = DashStyle.Dash;

        }

        /// <summary>
        /// Called on each bar update event (incoming tick)
        /// </summary>
        protected override void OnBarUpdate()
        {
			if (CurrentBar < rocLen) return;
			
            RocLine.Set(((Close[0] - Close[rocLen]) / Close[rocLen]) * 100.0);
            SigLine.Set(SMA(RocLine, signalLen)[0]);
        }

        #region Properties
        [Browsable(false)]	// this line prevents the data series from being displayed in the indicator properties dialog, do not remove
        [XmlIgnore()]		// this line ensures that the indicator can be saved/recovered as part of a chart template, do not remove
        public DataSeries RocLine
        {
            get { return Values[0]; }
        }

        [Browsable(false)]	// this line prevents the data series from being displayed in the indicator properties dialog, do not remove
        [XmlIgnore()]		// this line ensures that the indicator can be saved/recovered as part of a chart template, do not remove
        public DataSeries SigLine
        {
            get { return Values[1]; }
        }

        [Description("ROC Length")]
        [Category("Parameters")]
        public int RocLen
        {
            get { return rocLen; }
            set { rocLen = Math.Max(1, value); }
        }

        [Description("Signal Length")]
        [Category("Parameters")]
        public int SignalLen
        {
            get { return signalLen; }
            set { signalLen = Math.Max(1, value); }
        }
        #endregion
    }
}

#region NinjaScript generated code. Neither change nor remove.
// This namespace holds all indicators and is required. Do not change it.
namespace NinjaTrader.Indicator
{
    public partial class Indicator : IndicatorBase
    {
        private ROC1[] cacheROC1 = null;

        private static ROC1 checkROC1 = new ROC1();

        /// <summary>
        /// Rate of Change with Signal Line
        /// </summary>
        /// <returns></returns>
        public ROC1 ROC1(int rocLen, int signalLen)
        {
            return ROC1(Input, rocLen, signalLen);
        }

        /// <summary>
        /// Rate of Change with Signal Line
        /// </summary>
        /// <returns></returns>
        public ROC1 ROC1(Data.IDataSeries input, int rocLen, int signalLen)
        {
            checkROC1.RocLen = rocLen;
            rocLen = checkROC1.RocLen;
            checkROC1.SignalLen = signalLen;
            signalLen = checkROC1.SignalLen;

            if (cacheROC1 != null)
                for (int idx = 0; idx < cacheROC1.Length; idx++)
                    if (cacheROC1[idx].RocLen == rocLen && cacheROC1[idx].SignalLen == signalLen && cacheROC1[idx].EqualsInput(input))
                        return cacheROC1[idx];

            ROC1 indicator = new ROC1();
            indicator.BarsRequired = BarsRequired;
            indicator.CalculateOnBarClose = CalculateOnBarClose;
            indicator.Input = input;
            indicator.RocLen = rocLen;
            indicator.SignalLen = signalLen;
            indicator.SetUp();

            ROC1[] tmp = new ROC1[cacheROC1 == null ? 1 : cacheROC1.Length + 1];
            if (cacheROC1 != null)
                cacheROC1.CopyTo(tmp, 0);
            tmp[tmp.Length - 1] = indicator;
            cacheROC1 = tmp;
            Indicators.Add(indicator);

            return indicator;
        }

    }
}

// This namespace holds all market analyzer column definitions and is required. Do not change it.
namespace NinjaTrader.MarketAnalyzer
{
    public partial class Column : ColumnBase
    {
        /// <summary>
        /// Rate of Change with Signal Line
        /// </summary>
        /// <returns></returns>
        [Gui.Design.WizardCondition("Indicator")]
        public Indicator.ROC1 ROC1(int rocLen, int signalLen)
        {
            return _indicator.ROC1(Input, rocLen, signalLen);
        }

        /// <summary>
        /// Rate of Change with Signal Line
        /// </summary>
        /// <returns></returns>
        public Indicator.ROC1 ROC1(Data.IDataSeries input, int rocLen, int signalLen)
        {
            return _indicator.ROC1(input, rocLen, signalLen);
        }

    }
}

// This namespace holds all strategies and is required. Do not change it.
namespace NinjaTrader.Strategy
{
    public partial class Strategy : StrategyBase
    {
        /// <summary>
        /// Rate of Change with Signal Line
        /// </summary>
        /// <returns></returns>
        [Gui.Design.WizardCondition("Indicator")]
        public Indicator.ROC1 ROC1(int rocLen, int signalLen)
        {
            return _indicator.ROC1(Input, rocLen, signalLen);
        }

        /// <summary>
        /// Rate of Change with Signal Line
        /// </summary>
        /// <returns></returns>
        public Indicator.ROC1 ROC1(Data.IDataSeries input, int rocLen, int signalLen)
        {
            if (InInitialize && input == null)
                throw new ArgumentException("You only can access an indicator with the default input/bar series from within the 'Initialize()' method");

            return _indicator.ROC1(input, rocLen, signalLen);
        }

    }
}
#endregion
