#region Using declarations
using System;
using System.ComponentModel;
using System.Diagnostics;
using System.Drawing;
using System.Drawing.Drawing2D;
using System.Xml.Serialization;
using NinjaTrader.Cbi;
using NinjaTrader.Data;
using NinjaTrader.Gui.Chart;
#endregion

// This namespace holds all indicators and is required. Do not change it.
namespace NinjaTrader.Indicator
{
    /// <summary>
    /// Slow Stochastic %K Line
    /// </summary>
    [Description("Slow Stochastic %K Line using EMA smoothing")]
    public class SlowK : Indicator
    {
        #region Variables
        // Wizard generated variables
            private int kLen = 3; // Default setting for KLen
            private int smoothing = 8; // Default setting for EMA smoothing
        // User defined variables (add any user defined variables below)
            private const int dLen = 1; // %D Length is not used for the slow %K line
			private DataSeries fastK;

		#endregion

        /// <summary>
        /// This method is used to configure the indicator and is called once before any bar data is loaded.
        /// </summary>
        protected override void Initialize()
        {
            Add(new Plot(Color.FromKnownColor(KnownColor.White), PlotStyle.Line, "SlowKLine"));
            Add(new Line(Color.FromKnownColor(KnownColor.IndianRed), 90, "HighX"));
            Add(new Line(Color.FromKnownColor(KnownColor.Red), 75, "High"));
            Add(new Line(Color.FromKnownColor(KnownColor.LightSlateGray), 50, "MidLine"));
            Add(new Line(Color.FromKnownColor(KnownColor.LawnGreen), 25, "Low"));
            Add(new Line(Color.FromKnownColor(KnownColor.DarkOrchid), 10, "LowX"));

			CalculateOnBarClose	= true;
            Overlay				= false;
            PriceTypeSupported	= false;
			
			fastK = new DataSeries(this);
			
			Plots[0].Pen.Width = 2;
			Lines[0].Pen.DashStyle = DashStyle.Dot;
			Lines[1].Pen.DashStyle = DashStyle.Dot;
			Lines[2].Pen.DashStyle = DashStyle.Dash;
			Lines[3].Pen.DashStyle = DashStyle.Dot;
			Lines[4].Pen.DashStyle = DashStyle.Dot;
		}

        /// <summary>
        /// Called on each bar update event (incoming tick)
        /// </summary>
        protected override void OnBarUpdate()
        {
			fastK = StochasticsFast(dLen, kLen).K;
            SlowKLine.Set(EMA(fastK, smoothing)[0]);			
		}

        #region Properties
        [Browsable(false)]	// this line prevents the data series from being displayed in the indicator properties dialog, do not remove
        [XmlIgnore()]		// this line ensures that the indicator can be saved/recovered as part of a chart template, do not remove
        public DataSeries SlowKLine
        {
            get { return Values[0]; }
        }

        [Description("%K Length")]
        [Category("Parameters")]
        public int KLen
        {
            get { return kLen; }
            set { kLen = Math.Max(1, value); }
        }

        [Description("EMA Smoothing")]
        [Category("Parameters")]
        public int Smoothing
        {
            get { return smoothing; }
            set { smoothing = Math.Max(1, value); }
        }
        #endregion
    }
}

#region NinjaScript generated code. Neither change nor remove.
// This namespace holds all indicators and is required. Do not change it.
namespace NinjaTrader.Indicator
{
    public partial class Indicator : IndicatorBase
    {
        private SlowK[] cacheSlowK = null;

        private static SlowK checkSlowK = new SlowK();

        /// <summary>
        /// Slow Stochastic %K Line using EMA smoothing
        /// </summary>
        /// <returns></returns>
        public SlowK SlowK(int kLen, int smoothing)
        {
            return SlowK(Input, kLen, smoothing);
        }

        /// <summary>
        /// Slow Stochastic %K Line using EMA smoothing
        /// </summary>
        /// <returns></returns>
        public SlowK SlowK(Data.IDataSeries input, int kLen, int smoothing)
        {
            checkSlowK.KLen = kLen;
            kLen = checkSlowK.KLen;
            checkSlowK.Smoothing = smoothing;
            smoothing = checkSlowK.Smoothing;

            if (cacheSlowK != null)
                for (int idx = 0; idx < cacheSlowK.Length; idx++)
                    if (cacheSlowK[idx].KLen == kLen && cacheSlowK[idx].Smoothing == smoothing && cacheSlowK[idx].EqualsInput(input))
                        return cacheSlowK[idx];

            SlowK indicator = new SlowK();
            indicator.BarsRequired = BarsRequired;
            indicator.CalculateOnBarClose = CalculateOnBarClose;
            indicator.Input = input;
            indicator.KLen = kLen;
            indicator.Smoothing = smoothing;
            indicator.SetUp();

            SlowK[] tmp = new SlowK[cacheSlowK == null ? 1 : cacheSlowK.Length + 1];
            if (cacheSlowK != null)
                cacheSlowK.CopyTo(tmp, 0);
            tmp[tmp.Length - 1] = indicator;
            cacheSlowK = tmp;
            Indicators.Add(indicator);

            return indicator;
        }

    }
}

// This namespace holds all market analyzer column definitions and is required. Do not change it.
namespace NinjaTrader.MarketAnalyzer
{
    public partial class Column : ColumnBase
    {
        /// <summary>
        /// Slow Stochastic %K Line using EMA smoothing
        /// </summary>
        /// <returns></returns>
        [Gui.Design.WizardCondition("Indicator")]
        public Indicator.SlowK SlowK(int kLen, int smoothing)
        {
            return _indicator.SlowK(Input, kLen, smoothing);
        }

        /// <summary>
        /// Slow Stochastic %K Line using EMA smoothing
        /// </summary>
        /// <returns></returns>
        public Indicator.SlowK SlowK(Data.IDataSeries input, int kLen, int smoothing)
        {
            return _indicator.SlowK(input, kLen, smoothing);
        }

    }
}

// This namespace holds all strategies and is required. Do not change it.
namespace NinjaTrader.Strategy
{
    public partial class Strategy : StrategyBase
    {
        /// <summary>
        /// Slow Stochastic %K Line using EMA smoothing
        /// </summary>
        /// <returns></returns>
        [Gui.Design.WizardCondition("Indicator")]
        public Indicator.SlowK SlowK(int kLen, int smoothing)
        {
            return _indicator.SlowK(Input, kLen, smoothing);
        }

        /// <summary>
        /// Slow Stochastic %K Line using EMA smoothing
        /// </summary>
        /// <returns></returns>
        public Indicator.SlowK SlowK(Data.IDataSeries input, int kLen, int smoothing)
        {
            if (InInitialize && input == null)
                throw new ArgumentException("You only can access an indicator with the default input/bar series from within the 'Initialize()' method");

            return _indicator.SlowK(input, kLen, smoothing);
        }

    }
}
#endregion
